@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
-import os
-
 from calibre.customize.conversion import InputFormatPlugin
+from polyglot.builtins import getcwd
+
 
 class PDBInput(InputFormatPlugin):
 
     name        = 'PDB Input'
     author      = 'John Schember'
-    description = 'Convert PDB to HTML'
-    file_types  = set(['pdb', 'updb'])
+    description = _('Convert PDB to HTML')
+    file_types  = {'pdb', 'updb'}
+    commit_name = 'pdb_input'
 
     def convert(self, stream, options, file_ext, log,
                 accelerators):
@@ -24,11 +26,12 @@ class PDBInput(InputFormatPlugin):
         Reader = get_reader(header.ident)
 
         if Reader is None:
-            raise PDBError('No reader available for format within container.\n Identity is %s. Book type is %s' % (header.ident, IDENTITY_TO_NAME.get(header.ident, _('Unknown'))))
+            raise PDBError('No reader available for format within container.\n Identity is %s. Book type is %s' %
+                           (header.ident, IDENTITY_TO_NAME.get(header.ident, _('Unknown'))))
 
         log.debug('Detected ebook format as: %s with identity: %s' % (IDENTITY_TO_NAME[header.ident], header.ident))
 
         reader = Reader(header, stream, log, options)
-        opf = reader.extract_content(os.getcwdu())
+        opf = reader.extract_content(getcwd())
 
         return opf
