@@ -1,27 +1,21 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import weakref
+from qt.core import (
+    QLineEdit, QAbstractListModel, Qt, pyqtSignal, QObject, QKeySequence, QAbstractItemView,
+    QApplication, QListView, QPoint, QModelIndex, QEvent,
+    QStyleOptionComboBox, QStyle, QComboBox, QTimer, sip)
 
-from PyQt5.Qt import (
-    QLineEdit, QAbstractListModel, Qt, pyqtSignal, QObject, QKeySequence,
-    QApplication, QListView, QPoint, QModelIndex, QFont, QFontInfo,
-    QStyleOptionComboBox, QStyle, QComboBox, QTimer)
-try:
-    from PyQt5 import sip
-except ImportError:
-    import sip
-
-from calibre.constants import isosx, get_osx_version
+from calibre.constants import ismacos
 from calibre.utils.icu import sort_key, primary_startswith, primary_contains
 from calibre.gui2.widgets import EnComboBox, LineEditECM
 from calibre.utils.config import tweaks
+from polyglot.builtins import unicode_type
 
 
 def containsq(x, prefix):
@@ -30,15 +24,18 @@ def containsq(x, prefix):
 
 class CompleteModel(QAbstractListModel):  # {{{
 
-    def __init__(self, parent=None, sort_func=sort_key):
+    def __init__(self, parent=None, sort_func=sort_key, strip_completion_entries=True):
         QAbstractListModel.__init__(self, parent)
+        self.strip_completion_entries = strip_completion_entries
         self.sort_func = sort_func
         self.all_items = self.current_items = ()
         self.current_prefix = ''
 
     def set_items(self, items):
-        items = [unicode(x.strip()) for x in items]
-        items = [x for x in items if x]
+        if self.strip_completion_entries:
+            items = (str(x).strip() for x in items if x)
+        else:
+            items = (str(x) for x in items if x)
         items = tuple(sorted(items, key=self.sort_func))
         self.beginResetModel()
         self.all_items = self.current_items = items
@@ -66,7 +63,7 @@ class CompleteModel(QAbstractListModel):  # {{{
         return len(self.current_items)
 
     def data(self, index, role):
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             try:
                 return self.current_items[index.row()]
             except IndexError:
@@ -84,23 +81,23 @@ class Completer(QListView):  # {{{
     item_selected = pyqtSignal(object)
     relayout_needed = pyqtSignal()
 
-    def __init__(self, completer_widget, max_visible_items=7, sort_func=sort_key):
-        QListView.__init__(self)
+    def __init__(self, completer_widget, max_visible_items=7, sort_func=sort_key, strip_completion_entries=True):
+        QListView.__init__(self, completer_widget)
         self.disable_popup = False
-        self.completer_widget = weakref.ref(completer_widget)
-        self.setWindowFlags(Qt.Popup)
+        self.setWindowFlags(Qt.WindowType.Popup)
         self.max_visible_items = max_visible_items
-        self.setEditTriggers(self.NoEditTriggers)
-        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self.setSelectionBehavior(self.SelectRows)
-        self.setSelectionMode(self.SingleSelection)
+        self.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
+        self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
+        self.setUniformItemSizes(True)
         self.setAlternatingRowColors(True)
-        self.setModel(CompleteModel(self, sort_func=sort_func))
+        self.setModel(CompleteModel(self, sort_func=sort_func, strip_completion_entries=strip_completion_entries))
         self.setMouseTracking(True)
-        self.entered.connect(self.item_entered)
         self.activated.connect(self.item_chosen)
         self.pressed.connect(self.item_chosen)
         self.installEventFilter(self)
+        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
 
     def hide(self):
         self.setCurrentIndex(QModelIndex())
@@ -110,8 +107,8 @@ class Completer(QListView):  # {{{
         if not self.isVisible():
             return
         self.hide()
-        text = self.model().data(index, Qt.DisplayRole)
-        self.item_selected.emit(unicode(text))
+        text = self.model().data(index, Qt.ItemDataRole.DisplayRole)
+        self.item_selected.emit(unicode_type(text))
 
     def set_items(self, items):
         self.model().set_items(items)
@@ -123,16 +120,6 @@ class Completer(QListView):  # {{{
         if self.isVisible():
             self.relayout_needed.emit()
 
-    def item_entered(self, idx):
-        if self.visualRect(idx).top() < self.viewport().rect().bottom() - 5:
-            # Prevent any bottom item in the list that is only partially
-            # visible from triggering setCurrentIndex()
-            self.entered.disconnect()
-            try:
-                self.setCurrentIndex(idx)
-            finally:
-                self.entered.connect(self.item_entered)
-
     def next_match(self, previous=False):
         c = self.currentIndex()
         if c.isValid():
@@ -154,7 +141,7 @@ class Completer(QListView):  # {{{
             return
         p = self
         m = p.model()
-        widget = self.completer_widget()
+        widget = self.parent()
         if widget is None:
             return
         screen = QApplication.desktop().availableGeometry(widget)
@@ -188,25 +175,25 @@ class Completer(QListView):  # {{{
             self.setCurrentIndex(self.model().index(0))
 
         if not p.isVisible():
-            if isosx and get_osx_version() >= (10, 9, 0):
-                # On mavericks the popup menu seems to use a font smaller than
-                # the widgets font, see for example:
-                # https://bugs.launchpad.net/bugs/1243761
-                fp = QFontInfo(widget.font())
-                f = QFont()
-                f.setPixelSize(fp.pixelSize())
-                self.setFont(f)
             p.show()
 
     def debug_event(self, ev):
         from calibre.gui2 import event_type_name
         print('Event:', event_type_name(ev))
-        if ev.type() in (ev.KeyPress, ev.ShortcutOverride, ev.KeyRelease):
+        if ev.type() in (QEvent.Type.KeyPress, QEvent.Type.ShortcutOverride, QEvent.Type.KeyRelease):
             print('\tkey:', QKeySequence(ev.key()).toString())
 
+    def mouseMoveEvent(self, ev):
+        idx = self.indexAt(ev.pos())
+        if idx.isValid():
+            ci = self.currentIndex()
+            if idx.row() != ci.row():
+                self.setCurrentIndex(idx)
+        return QListView.mouseMoveEvent(self, ev)
+
     def eventFilter(self, obj, e):
         'Redirect key presses from the popup to the widget'
-        widget = self.completer_widget()
+        widget = self.parent()
         if widget is None or sip.isdeleted(widget):
             return False
         etype = e.type()
@@ -215,20 +202,20 @@ class Completer(QListView):  # {{{
 
         # self.debug_event(e)
 
-        if etype == e.KeyPress:
+        if etype == QEvent.Type.KeyPress:
             try:
                 key = e.key()
             except AttributeError:
                 return QObject.eventFilter(self, obj, e)
-            if key == Qt.Key_Escape:
+            if key == Qt.Key.Key_Escape:
                 self.hide()
                 e.accept()
                 return True
-            if key == Qt.Key_F4 and e.modifiers() & Qt.AltModifier:
+            if key == Qt.Key.Key_F4 and e.modifiers() & Qt.KeyboardModifier.AltModifier:
                 self.hide()
                 e.accept()
                 return True
-            if key in (Qt.Key_Enter, Qt.Key_Return):
+            if key in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
                 # We handle this explicitly because on OS X activated() is
                 # not emitted on pressing Enter.
                 idx = self.currentIndex()
@@ -237,7 +224,7 @@ class Completer(QListView):  # {{{
                 self.hide()
                 e.accept()
                 return True
-            if key == Qt.Key_Tab:
+            if key == Qt.Key.Key_Tab:
                 idx = self.currentIndex()
                 if idx.isValid():
                     self.item_chosen(idx)
@@ -246,11 +233,11 @@ class Completer(QListView):  # {{{
                     self.next_match()
                 e.accept()
                 return True
-            if key in (Qt.Key_PageUp, Qt.Key_PageDown):
+            if key in (Qt.Key.Key_PageUp, Qt.Key.Key_PageDown):
                 # Let the list view handle these keys
                 return False
-            if key in (Qt.Key_Up, Qt.Key_Down):
-                self.next_match(previous=key == Qt.Key_Up)
+            if key in (Qt.Key.Key_Up, Qt.Key.Key_Down):
+                self.next_match(previous=key == Qt.Key.Key_Up)
                 e.accept()
                 return True
             # Send to widget
@@ -262,28 +249,29 @@ class Completer(QListView):  # {{{
                 self.hide()
             if e.isAccepted():
                 return True
-        elif isosx and etype == e.InputMethodQuery and e.queries() == (Qt.ImHints | Qt.ImEnabled) and self.isVisible():
+        elif ismacos and etype == QEvent.Type.InputMethodQuery and e.queries() == (
+            Qt.InputMethodQuery.ImHints | Qt.InputMethodQuery.ImEnabled) and self.isVisible():
             # In Qt 5 the Esc key causes this event and the line edit does not
             # handle it, which causes the parent dialog to be closed
             # See https://bugreports.qt-project.org/browse/QTBUG-41806
             e.accept()
             return True
-        elif etype == e.MouseButtonPress and hasattr(e, 'globalPos') and not self.rect().contains(self.mapFromGlobal(e.globalPos())):
+        elif etype == QEvent.Type.MouseButtonPress and hasattr(e, 'globalPos') and not self.rect().contains(self.mapFromGlobal(e.globalPos())):
             # A click outside the popup, close it
             if isinstance(widget, QComboBox):
                 # This workaround is needed to ensure clicking on the drop down
                 # arrow of the combobox closes the popup
                 opt = QStyleOptionComboBox()
                 widget.initStyleOption(opt)
-                sc = widget.style().hitTestComplexControl(QStyle.CC_ComboBox, opt, widget.mapFromGlobal(e.globalPos()), widget)
-                if sc == QStyle.SC_ComboBoxArrow:
+                sc = widget.style().hitTestComplexControl(QStyle.ComplexControl.CC_ComboBox, opt, widget.mapFromGlobal(e.globalPos()), widget)
+                if sc == QStyle.SubControl.SC_ComboBoxArrow:
                     QTimer.singleShot(0, self.hide)
                     e.accept()
                     return True
             self.hide()
             e.accept()
             return True
-        elif etype in (e.InputMethod, e.ShortcutOverride):
+        elif etype in (QEvent.Type.InputMethod, QEvent.Type.ShortcutOverride):
             QApplication.sendEvent(widget, e)
         return False
 # }}}
@@ -301,7 +289,7 @@ class LineEdit(QLineEdit, LineEditECM):
     '''
     item_selected = pyqtSignal(object)
 
-    def __init__(self, parent=None, completer_widget=None, sort_func=sort_key):
+    def __init__(self, parent=None, completer_widget=None, sort_func=sort_key, strip_completion_entries=True):
         QLineEdit.__init__(self, parent)
 
         self.sep = ','
@@ -311,9 +299,9 @@ class LineEdit(QLineEdit, LineEditECM):
         completer_widget = (self if completer_widget is None else
                 completer_widget)
 
-        self.mcompleter = Completer(completer_widget, sort_func=sort_func)
+        self.mcompleter = Completer(completer_widget, sort_func=sort_func, strip_completion_entries=strip_completion_entries)
         self.mcompleter.item_selected.connect(self.completion_selected,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.mcompleter.relayout_needed.connect(self.relayout)
         self.mcompleter.setFocusProxy(completer_widget)
         self.textEdited.connect(self.text_edited)
@@ -332,30 +320,35 @@ class LineEdit(QLineEdit, LineEditECM):
     def set_add_separator(self, what):
         self.add_separator = bool(what)
 
-    @dynamic_property
+    @property
     def all_items(self):
-        def fget(self):
-            return self.mcompleter.model().all_items
+        return self.mcompleter.model().all_items
 
-        def fset(self, items):
-            self.mcompleter.model().set_items(items)
-        return property(fget=fget, fset=fset)
+    @all_items.setter
+    def all_items(self, items):
+        self.mcompleter.model().set_items(items)
 
-    @dynamic_property
+    @property
     def disable_popup(self):
-        def fget(self):
-            return self.mcompleter.disable_popup
+        return self.mcompleter.disable_popup
 
-        def fset(self, val):
-            self.mcompleter.disable_popup = bool(val)
-        return property(fget=fget, fset=fset)
+    @disable_popup.setter
+    def disable_popup(self, val):
+        self.mcompleter.disable_popup = bool(val)
+
+    def set_elide_mode(self, val):
+        self.mcompleter.setTextElideMode(val)
     # }}}
 
     def event(self, ev):
         # See https://bugreports.qt.io/browse/QTBUG-46911
-        if ev.type() == ev.ShortcutOverride and (
-                ev.key() in (Qt.Key_Left, Qt.Key_Right) and (ev.modifiers() & ~Qt.KeypadModifier) == Qt.ControlModifier):
-            ev.accept()
+        try:
+            if ev.type() == QEvent.Type.ShortcutOverride and (
+                    ev.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right) and (
+                        ev.modifiers() & ~Qt.KeyboardModifier.KeypadModifier) == Qt.KeyboardModifier.ControlModifier):
+                ev.accept()
+        except AttributeError:
+            pass
         return QLineEdit.event(self, ev)
 
     def complete(self, show_all=False, select_first=True):
@@ -367,12 +360,12 @@ class LineEdit(QLineEdit, LineEditECM):
             self.mcompleter.hide()
             return
         self.mcompleter.popup(select_first=select_first)
-        self.setFocus(Qt.OtherFocusReason)
+        self.setFocus(Qt.FocusReason.OtherFocusReason)
         self.mcompleter.scroll_to(orig)
 
     def relayout(self):
         self.mcompleter.popup()
-        self.setFocus(Qt.OtherFocusReason)
+        self.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def text_edited(self, *args):
         if self.no_popup:
@@ -386,7 +379,7 @@ class LineEdit(QLineEdit, LineEditECM):
     def update_completions(self):
         ' Update the list of completions '
         self.original_cursor_pos = cpos = self.cursorPosition()
-        text = unicode(self.text())
+        text = unicode_type(self.text())
         prefix = text[:cpos]
         complete_prefix = prefix.lstrip()
         if self.sep:
@@ -403,7 +396,7 @@ class LineEdit(QLineEdit, LineEditECM):
                 cursor_pos = self.cursorPosition()
             self.original_cursor_pos = None
             # Split text
-            curtext = unicode(self.text())
+            curtext = unicode_type(self.text())
             before_text = curtext[:cursor_pos]
             after_text = curtext[cursor_pos:].rstrip()
             # Remove the completion prefix from the before text
@@ -423,7 +416,7 @@ class LineEdit(QLineEdit, LineEditECM):
             return before_text + completed_text, after_text
 
     def completion_selected(self, text):
-        before_text, after_text = self.get_completed_text(unicode(text))
+        before_text, after_text = self.get_completed_text(unicode_type(text))
         self.setText(before_text + after_text)
         self.setCursorPosition(len(before_text))
         self.item_selected.emit(text)
@@ -435,7 +428,9 @@ class EditWithComplete(EnComboBox):
 
     def __init__(self, *args, **kwargs):
         EnComboBox.__init__(self, *args)
-        self.setLineEdit(LineEdit(self, completer_widget=self, sort_func=kwargs.get('sort_func', sort_key)))
+        self.setLineEdit(LineEdit(
+            self, completer_widget=self, sort_func=kwargs.get('sort_func', sort_key),
+            strip_completion_entries=kwargs.get('strip_completion_entries', False)))
         self.lineEdit().item_selected.connect(self.item_selected)
         self.setCompleter(None)
         self.eat_focus_out = True
@@ -463,31 +458,32 @@ class EditWithComplete(EnComboBox):
         self.lineEdit().set_add_separator(what)
 
     def show_initial_value(self, what):
-        what = unicode(what) if what else u''
+        what = unicode_type(what) if what else u''
         self.setText(what)
         self.lineEdit().selectAll()
 
-    @dynamic_property
+    @property
     def all_items(self):
-        def fget(self):
-            return self.lineEdit().all_items
+        return self.lineEdit().all_items
 
-        def fset(self, val):
-            self.lineEdit().all_items = val
-        return property(fget=fget, fset=fset)
+    @all_items.setter
+    def all_items(self, val):
+        self.lineEdit().all_items = val
 
-    @dynamic_property
+    @property
     def disable_popup(self):
-        def fget(self):
-            return self.lineEdit().disable_popup
+        return self.lineEdit().disable_popup
 
-        def fset(self, val):
-            self.lineEdit().disable_popup = bool(val)
-        return property(fget=fget, fset=fset)
+    @disable_popup.setter
+    def disable_popup(self, val):
+        self.lineEdit().disable_popup = bool(val)
+
+    def set_elide_mode(self, val):
+        self.lineEdit().set_elide_mode(val)
     # }}}
 
     def text(self):
-        return unicode(self.lineEdit().text())
+        return unicode_type(self.lineEdit().text())
 
     def selectAll(self):
         self.lineEdit().selectAll()
@@ -498,6 +494,9 @@ class EditWithComplete(EnComboBox):
         le.setText(val)
         le.no_popup = False
 
+    def home(self, mark=False):
+        self.lineEdit().home(mark)
+
     def setCursorPosition(self, *args):
         self.lineEdit().setCursorPosition(*args)
 
@@ -515,20 +514,21 @@ class EditWithComplete(EnComboBox):
         except AttributeError:
             return False
         etype = e.type()
-        if self.eat_focus_out and self is obj and etype == e.FocusOut:
+        if self.eat_focus_out and self is obj and etype == QEvent.Type.FocusOut:
             if c.isVisible():
                 return True
         return EnComboBox.eventFilter(self, obj, e)
 
 
 if __name__ == '__main__':
-    from PyQt5.Qt import QDialog, QVBoxLayout
-    app = QApplication([])
+    from qt.core import QDialog, QVBoxLayout
+    from calibre.gui2 import Application
+    app = Application([])
     d = QDialog()
     d.setLayout(QVBoxLayout())
     le = EditWithComplete(d)
     d.layout().addWidget(le)
-    items = ['one', 'otwo', 'othree', 'ooone', 'ootwo',
+    items = ['one', 'otwo', 'othree', 'ooone', 'ootwo', 'other', 'odd', 'over', 'orc', 'oven', 'owe',
         'oothree', 'a1', 'a2',u'Edgas', u'Èdgar', u'Édgaq', u'Edgar', u'Édgar']
     le.update_items_cache(items)
     le.show_initial_value('')
