@@ -5,8 +5,9 @@
 import codecs
 import json
 import os
-from functools import partial
-from PyQt5.Qt import (
+import re
+from functools import lru_cache, partial
+from qt.core import (
     QAbstractItemView, QApplication, QCheckBox, QComboBox, QCursor, QDateTime,
     QDialog, QDialogButtonBox, QFont, QFormLayout, QFrame, QHBoxLayout, QIcon,
     QKeySequence, QLabel, QMenu, QPalette, QPlainTextEdit, QSize, QSplitter, Qt,
@@ -16,15 +17,17 @@ from PyQt5.Qt import (
 from urllib.parse import quote
 
 from calibre import prepare_string_for_xml
+from calibre.db.backend import FTSQueryError
 from calibre.ebooks.metadata import authors_to_string, fmt_sidx
-from calibre.gui2 import Application, choose_save_file, config, error_dialog, gprefs
+from calibre.gui2 import (
+    Application, choose_save_file, config, error_dialog, gprefs, safe_open_url
+)
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.viewer.widgets import ResultsDelegate, SearchBox
 from calibre.gui2.widgets2 import Dialog
 
 
 # rendering {{{
-
 def render_highlight_as_text(hl, lines, as_markdown=False, link_prefix=None):
     lines.append(hl['highlighted_text'])
     date = QDateTime.fromString(hl['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
@@ -61,11 +64,57 @@ def render_bookmark_as_text(b, lines, as_markdown=False, link_prefix=None):
     lines.append('')
 
 
+url_prefixes = 'http', 'https'
+url_delimiters = (
+    '\x00-\x09\x0b-\x20\x7f-\xa0\xad\u0600-\u0605\u061c\u06dd\u070f\u08e2\u1680\u180e\u2000-\u200f\u2028-\u202f'
+    '\u205f-\u2064\u2066-\u206f\u3000\ud800-\uf8ff\ufeff\ufff9-\ufffb\U000110bd\U000110cd\U00013430-\U00013438'
+    '\U0001bca0-\U0001bca3\U0001d173-\U0001d17a\U000e0001\U000e0020-\U000e007f\U000f0000-\U000ffffd\U00100000-\U0010fffd'
+)
+url_pattern = r'\b(?:{})://[^{}]{{3,}}'.format('|'.join(url_prefixes), url_delimiters)
+
+
+@lru_cache(maxsize=2)
+def url_pat():
+    return re.compile(url_pattern, flags=re.I)
+
+
+closing_bracket_map = {'(': ')', '[': ']', '{': '}', '<': '>', '*': '*', '"': '"', "'": "'"}
+
+
+def url(text: str, s: int, e: int):
+    while text[e - 1] in '.,?!' and e > 1:  # remove trailing punctuation
+        e -= 1
+    # truncate url at closing bracket/quote
+    if s > 0 and e <= len(text) and text[s-1] in closing_bracket_map:
+        q = closing_bracket_map[text[s-1]]
+        idx = text.find(q, s)
+        if idx > s:
+            e = idx
+    return s, e
+
+
+def render_note_line(line):
+    urls = []
+    for m in url_pat().finditer(line):
+        s, e = url(line, m.start(), m.end())
+        urls.append((s, e))
+    if not urls:
+        yield prepare_string_for_xml(line)
+        return
+    pos = 0
+    for (s, e) in urls:
+        if s > pos:
+            yield prepare_string_for_xml(line[pos:s])
+        yield '<a href="{0}">{0}</a>'.format(prepare_string_for_xml(line[s:e], True))
+    if urls[-1][1] < len(line):
+        yield prepare_string_for_xml(line[urls[-1][1]:])
+
+
 def render_notes(notes, tag='p'):
     current_lines = []
     for line in notes.splitlines():
         if line:
-            current_lines.append(line)
+            current_lines.append(''.join(render_note_line(line)))
         else:
             if current_lines:
                 yield '<{0}>{1}</{0}>'.format(tag, '\n'.join(current_lines))
@@ -113,6 +162,28 @@ class AnnotsResultsDelegate(ResultsDelegate):
 # }}}
 
 
+def sorted_items(items):
+    from calibre.ebooks.epub.cfi.parse import cfi_sort_key
+    def_spine = 999999999
+    defval = cfi_sort_key(f'/{def_spine}')
+
+    def sort_key(x):
+        x = x['annotation']
+        atype = x['type']
+        if atype == 'highlight':
+            cfi = x.get('start_cfi')
+            if cfi:
+                spine_idx = x.get('spine_index', def_spine)
+                cfi = f'/{spine_idx}{cfi}'
+                return cfi_sort_key(cfi)
+        elif atype == 'bookmark':
+            if x.get('pos_type') == 'epubcfi':
+                return cfi_sort_key(x['pos'], only_path=False)
+        return defval
+
+    return sorted(items, key=sort_key)
+
+
 class Export(Dialog):  # {{{
 
     prefs = gprefs
@@ -157,10 +228,10 @@ class Export(Dialog):  # {{{
         self.accept()
 
     def save_to_file(self):
-        filters = [(self.export_format.currentText(), self.export_format.currentData())]
+        filters = [(self.export_format.currentText(), [self.export_format.currentData()])]
         path = choose_save_file(
             self, 'annots-export-save', _('File for exports'), filters=filters,
-            initial_filename=self.initial_filename() + '.' + filters[0][1])
+            initial_filename=self.initial_filename() + '.' + filters[0][1][0])
         if path:
             data = self.exported_data().encode('utf-8')
             with open(path, 'wb') as f:
@@ -186,18 +257,29 @@ class Export(Dialog):  # {{{
         for a in self.annotations:
             bid_groups.setdefault(a['book_id'], []).append(a)
         for book_id, group in bid_groups.items():
+            chapter_groups = {}
+            def_chap = (_('Unknown chapter'),)
+            for a in group:
+                toc_titles = a.get('toc_family_titles', def_chap)
+                chapter_groups.setdefault(toc_titles[0], []).append(a)
+
             lines.append('## ' + db.field_for('title', book_id))
             lines.append('')
-            for a in group:
-                atype = a['type']
-                if library_id:
-                    link_prefix = f'calibre://view-book/{library_id}/{book_id}/{a["format"]}?open_at='
-                else:
-                    link_prefix = None
-                if atype == 'highlight':
-                    render_highlight_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
-                elif atype == 'bookmark':
-                    render_bookmark_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
+
+            for chapter, group in chapter_groups.items():
+                if len(chapter_groups) > 1:
+                    lines.append('### ' + chapter)
+                    lines.append('')
+                for a in group:
+                    atype = a['type']
+                    if library_id:
+                        link_prefix = f'calibre://view-book/{library_id}/{book_id}/{a["format"]}?open_at='
+                    else:
+                        link_prefix = None
+                    if atype == 'highlight':
+                        render_highlight_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
+                    elif atype == 'bookmark':
+                        render_bookmark_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
             lines.append('')
         return '\n'.join(lines).strip()
 # }}}
@@ -294,9 +376,10 @@ class ResultsList(QTreeWidget):
             section = QTreeWidgetItem([entry['title']], 1)
             section.setFlags(Qt.ItemFlag.ItemIsEnabled)
             section.setFont(0, self.section_font)
+            section.setData(0, Qt.ItemDataRole.UserRole, book_id)
             self.addTopLevelItem(section)
             section.setExpanded(True)
-            for result in entry['matches']:
+            for result in sorted_items(entry['matches']):
                 item = QTreeWidgetItem(section, [' '], 2)
                 self.item_map.append(item)
                 item.setFlags(Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemNeverHasChildren)
@@ -342,8 +425,39 @@ class ResultsList(QTreeWidget):
             self.delete_requested.emit()
             ev.accept()
             return
+        if ev.key() == Qt.Key.Key_F2:
+            item = self.currentItem()
+            if item:
+                self.edit_notes(item)
+                ev.accept()
+                return
         return QTreeWidget.keyPressEvent(self, ev)
 
+    @property
+    def tree_state(self):
+        ans = {'closed': set()}
+        item = self.currentItem()
+        if item is not None:
+            ans['current'] = item.data(0, Qt.ItemDataRole.UserRole)
+        for item in (self.topLevelItem(i) for i in range(self.topLevelItemCount())):
+            if not item.isExpanded():
+                ans['closed'].add(item.data(0, Qt.ItemDataRole.UserRole))
+        return ans
+
+    @tree_state.setter
+    def tree_state(self, state):
+        closed = state['closed']
+        for item in (self.topLevelItem(i) for i in range(self.topLevelItemCount())):
+            if item.data(0, Qt.ItemDataRole.UserRole) in closed:
+                item.setExpanded(False)
+
+        cur = state.get('current')
+        if cur is not None:
+            for item in self.item_map:
+                if item.data(0, Qt.ItemDataRole.UserRole) == cur:
+                    self.setCurrentItem(item)
+                    break
+
 
 class Restrictions(QWidget):
 
@@ -543,22 +657,29 @@ class BrowsePanel(QWidget):
         if q == self.current_query:
             self.results_list.show_next(backwards)
             return
-        with BusyCursor():
-            db = current_db()
-            if not q['fts_engine_query']:
-                results = db.all_annotations(
-                    restrict_to_user=q['restrict_to_user'], limit=4096, annotation_type=q['annotation_type'],
-                    ignore_removed=True, restrict_to_book_ids=q['restrict_to_book_ids'] or None
-                )
-            else:
-                q2 = q.copy()
-                q2['restrict_to_book_ids'] = q.get('restrict_to_book_ids') or None
-                results = db.search_annotations(
-                    highlight_start='\x1d', highlight_end='\x1d', snippet_size=64,
-                    ignore_removed=True, **q2
-                )
-            self.results_list.set_results(results, bool(q['fts_engine_query']))
-            self.current_query = q
+        try:
+            with BusyCursor():
+                db = current_db()
+                if not q['fts_engine_query']:
+                    results = db.all_annotations(
+                        restrict_to_user=q['restrict_to_user'], limit=4096, annotation_type=q['annotation_type'],
+                        ignore_removed=True, restrict_to_book_ids=q['restrict_to_book_ids'] or None
+                    )
+                else:
+                    q2 = q.copy()
+                    q2['restrict_to_book_ids'] = q.get('restrict_to_book_ids') or None
+                    results = db.search_annotations(
+                        highlight_start='\x1d', highlight_end='\x1d', snippet_size=64,
+                        ignore_removed=True, **q2
+                    )
+                self.results_list.set_results(results, bool(q['fts_engine_query']))
+                self.current_query = q
+        except FTSQueryError as err:
+            return error_dialog(self, _('Invalid search expression'), '<p>' + _(
+                'The search expression: {0} is invalid. The search syntax used is the'
+                ' SQLite Full text Search Query syntax, <a href="{1}">described here</a>.').format(
+                    err.query, 'https://www.sqlite.org/fts5.html#full_text_query_syntax'),
+                det_msg=str(err), show=True)
 
     def effective_query_changed(self):
         self.do_find()
@@ -587,6 +708,12 @@ class BrowsePanel(QWidget):
     def selected_annotations(self):
         return self.results_list.selected_annotations
 
+    def save_tree_state(self):
+        return self.results_list.tree_state
+
+    def restore_tree_state(self, state):
+        self.results_list.tree_state = state
+
 
 class Details(QTextBrowser):
 
@@ -618,7 +745,10 @@ class DetailsPanel(QWidget):
         self.show_result(None)
 
     def link_clicked(self, qurl):
-        getattr(self, qurl.host())()
+        if qurl.scheme() == 'calibre':
+            getattr(self, qurl.host())()
+        else:
+            safe_open_url(qurl)
 
     def open_result(self):
         if self.current_result is not None:
@@ -679,7 +809,8 @@ class DetailsPanel(QWidget):
         if annot['type'] == 'bookmark':
             p(annot['title'])
         elif annot['type'] == 'highlight':
-            p(annot['highlighted_text'])
+            for line in annot['highlighted_text'].splitlines():
+                p(line)
             notes = annot.get('notes')
             if notes:
                 paras.append('<h4>{} (<a title="{}" href="calibre://edit_result">{}</a>)</h4>'.format(
@@ -723,7 +854,8 @@ class EditNotes(Dialog):
 
     def __init__(self, notes, parent=None):
         self.initial_notes = notes
-        Dialog.__init__(self, _('Edit notes for highlight'), 'library-annotations-browser-edit-notes', parent=parent)
+        Dialog.__init__(
+            self, _('Edit notes for highlight'), 'library-annotations-browser-edit-notes', parent=parent)
 
     def setup_ui(self):
         self.notes_edit = QPlainTextEdit(self)
@@ -746,7 +878,8 @@ class AnnotationsBrowser(Dialog):
     show_book = pyqtSignal(object, object)
 
     def __init__(self, parent=None):
-        Dialog.__init__(self, _('Annotations browser'), 'library-annotations-browser', parent=parent)
+        self.current_restriction = None
+        Dialog.__init__(self, _('Annotations browser'), 'library-annotations-browser', parent=parent, default_buttons=QDialogButtonBox.StandardButton.Close)
         self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
         self.setWindowIcon(QIcon(I('highlight.png')))
 
@@ -804,6 +937,10 @@ class AnnotationsBrowser(Dialog):
         b.setToolTip(_('Export the selected annotations'))
         b.setIcon(QIcon(I('save.png')))
         b.clicked.connect(self.export_selected)
+        self.refresh_button = b = self.bb.addButton(_('Refresh'), QDialogButtonBox.ButtonRole.ActionRole)
+        b.setToolTip(_('Refresh annotations in case they have been changed since this window was opened'))
+        b.setIcon(QIcon(I('restart.png')))
+        b.clicked.connect(self.refresh)
 
     def delete_selected(self):
         ids = frozenset(self.browse_panel.selected_annot_ids)
@@ -864,8 +1001,15 @@ class AnnotationsBrowser(Dialog):
             self.browse_panel.selection_changed(gui.library_view.get_selected_ids(as_set=True))
 
     def reinitialize(self, restrict_to_book_ids=None):
+        self.current_restriction = restrict_to_book_ids
         self.browse_panel.re_initialize(restrict_to_book_ids or set())
 
+    def refresh(self, current_restriction):
+        state = self.browse_panel.save_tree_state()
+        self.browse_panel.re_initialize(self.current_restriction)
+        self.browse_panel.effective_query_changed()
+        self.browse_panel.restore_tree_state(state)
+
 
 if __name__ == '__main__':
     from calibre.library import db
