@@ -10,9 +10,9 @@ import itertools, operator
 from functools import partial
 from collections import OrderedDict
 
-from PyQt5.Qt import (
+from qt.core import (
     QTableView, Qt, QAbstractItemView, QMenu, pyqtSignal, QFont, QModelIndex,
-    QIcon, QItemSelection, QMimeData, QDrag, QStyle, QPoint, QUrl, QHeaderView,
+    QIcon, QItemSelection, QMimeData, QDrag, QStyle, QPoint, QUrl, QHeaderView, QEvent,
     QStyleOptionHeader, QItemSelectionModel, QSize, QFontMetrics, QApplication)
 
 from calibre.constants import islinux
@@ -48,10 +48,11 @@ class HeaderView(QHeaderView):  # {{{
 
     def __init__(self, *args):
         QHeaderView.__init__(self, *args)
-        if self.orientation() == Qt.Horizontal:
+        if self.orientation() == Qt.Orientation.Horizontal:
             self.setSectionsMovable(True)
             self.setSectionsClickable(True)
-            self.setContextMenuPolicy(Qt.CustomContextMenu)
+            self.setTextElideMode(Qt.TextElideMode.ElideRight)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.hover = -1
         self.current_font = QFont(self.font())
         self.current_font.setBold(True)
@@ -59,9 +60,9 @@ class HeaderView(QHeaderView):  # {{{
         self.fm = QFontMetrics(self.current_font)
 
     def event(self, e):
-        if e.type() in (e.HoverMove, e.HoverEnter):
+        if e.type() in (QEvent.Type.HoverMove, QEvent.Type.HoverEnter):
             self.hover = self.logicalIndexAt(e.pos())
-        elif e.type() in (e.Leave, e.HoverLeave):
+        elif e.type() in (QEvent.Type.Leave, QEvent.Type.HoverLeave):
             self.hover = -1
         return QHeaderView.event(self, e)
 
@@ -73,18 +74,18 @@ class HeaderView(QHeaderView):  # {{{
         opt.orientation = self.orientation()
         opt.fontMetrics = self.fm
         model = self.parent().model()
-        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.DisplayRole) or '')
-        if opt.orientation == Qt.Vertical:
+        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
+        if opt.orientation == Qt.Orientation.Vertical:
             try:
-                val = model.headerData(logical_index, opt.orientation, Qt.DecorationRole)
+                val = model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DecorationRole)
                 if val is not None:
                     opt.icon = val
-                opt.iconAlignment = Qt.AlignVCenter
+                opt.iconAlignment = Qt.AlignmentFlag.AlignVCenter
             except (IndexError, ValueError, TypeError):
                 pass
         if self.isSortIndicatorShown():
-            opt.sortIndicator = QStyleOptionHeader.SortDown
-        return self.style().sizeFromContents(QStyle.CT_HeaderSection, opt, QSize(), self)
+            opt.sortIndicator = QStyleOptionHeader.SortIndicator.SortDown
+        return self.style().sizeFromContents(QStyle.ContentsType.CT_HeaderSection, opt, QSize(), self)
 
     def paintSection(self, painter, rect, logical_index):
         opt = QStyleOptionHeader()
@@ -92,41 +93,42 @@ class HeaderView(QHeaderView):  # {{{
         opt.rect = rect
         opt.section = logical_index
         opt.orientation = self.orientation()
-        opt.textAlignment = Qt.AlignHCenter | Qt.AlignVCenter
+        opt.textAlignment = Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter
         opt.fontMetrics = self.fm
         model = self.parent().model()
         style = self.style()
-        margin = 2 * style.pixelMetric(style.PM_HeaderMargin, None, self)
+        margin = 2 * style.pixelMetric(QStyle.PixelMetric.PM_HeaderMargin, None, self)
         if self.isSortIndicatorShown() and self.sortIndicatorSection() == logical_index:
-            opt.sortIndicator = QStyleOptionHeader.SortDown if self.sortIndicatorOrder() == Qt.AscendingOrder else QStyleOptionHeader.SortUp
-            margin += style.pixelMetric(style.PM_HeaderMarkSize, None, self)
-        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.DisplayRole) or '')
-        if self.textElideMode() != Qt.ElideNone:
-            opt.text = opt.fontMetrics.elidedText(opt.text, Qt.ElideRight, rect.width() - margin)
+            opt.sortIndicator = QStyleOptionHeader.SortIndicator.SortDown if \
+                self.sortIndicatorOrder() == Qt.SortOrder.AscendingOrder else QStyleOptionHeader.SortIndicator.SortUp
+            margin += style.pixelMetric(QStyle.PixelMetric.PM_HeaderMarkSize, None, self)
+        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
+        if self.textElideMode() != Qt.TextElideMode.ElideNone:
+            opt.text = opt.fontMetrics.elidedText(opt.text, Qt.TextElideMode.ElideRight, rect.width() - margin)
         if self.isEnabled():
-            opt.state |= QStyle.State_Enabled
+            opt.state |= QStyle.StateFlag.State_Enabled
             if self.window().isActiveWindow():
-                opt.state |= QStyle.State_Active
+                opt.state |= QStyle.StateFlag.State_Active
                 if self.hover == logical_index:
-                    opt.state |= QStyle.State_MouseOver
+                    opt.state |= QStyle.StateFlag.State_MouseOver
         sm = self.selectionModel()
-        if opt.orientation == Qt.Vertical:
+        if opt.orientation == Qt.Orientation.Vertical:
             try:
-                val = model.headerData(logical_index, opt.orientation, Qt.DecorationRole)
+                val = model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DecorationRole)
                 if val is not None:
                     opt.icon = val
-                opt.iconAlignment = Qt.AlignVCenter
+                opt.iconAlignment = Qt.AlignmentFlag.AlignVCenter
             except (IndexError, ValueError, TypeError):
                 pass
             if sm.isRowSelected(logical_index, QModelIndex()):
-                opt.state |= QStyle.State_Sunken
+                opt.state |= QStyle.StateFlag.State_Sunken
 
         painter.save()
         if (
-                (opt.orientation == Qt.Horizontal and sm.currentIndex().column() == logical_index) or (
-                    opt.orientation == Qt.Vertical and sm.currentIndex().row() == logical_index)):
+                (opt.orientation == Qt.Orientation.Horizontal and sm.currentIndex().column() == logical_index) or (
+                    opt.orientation == Qt.Orientation.Vertical and sm.currentIndex().row() == logical_index)):
             painter.setFont(self.current_font)
-        self.style().drawControl(QStyle.CE_Header, opt, painter, self)
+        self.style().drawControl(QStyle.ControlElement.CE_Header, opt, painter, self)
         painter.restore()
 # }}}
 
@@ -206,11 +208,12 @@ class BooksView(QTableView):  # {{{
 
     files_dropped = pyqtSignal(object)
     books_dropped = pyqtSignal(object)
+    selection_changed = pyqtSignal()
     add_column_signal = pyqtSignal()
     is_library_view = True
 
     def viewportEvent(self, event):
-        if (event.type() == event.ToolTip and not gprefs['book_list_tooltips']):
+        if (event.type() == QEvent.Type.ToolTip and not gprefs['book_list_tooltips']):
             return False
         try:
             ret = self.gesture_manager.handle_event(event)
@@ -233,15 +236,19 @@ class BooksView(QTableView):  # {{{
 
         for wv in self, self.pin_view:
             if not tweaks['horizontal_scrolling_per_column']:
-                wv.setHorizontalScrollMode(self.ScrollPerPixel)
+                wv.setHorizontalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
 
-            wv.setEditTriggers(self.EditKeyPressed)
-            if tweaks['doubleclick_on_library_view'] == 'edit_cell':
-                wv.setEditTriggers(self.DoubleClicked|wv.editTriggers())
-            elif tweaks['doubleclick_on_library_view'] == 'open_viewer':
-                wv.setEditTriggers(self.SelectedClicked|wv.editTriggers())
+            wv.setEditTriggers(QAbstractItemView.EditTrigger.EditKeyPressed)
+            tval = tweaks['doubleclick_on_library_view']
+            if tval == 'edit_cell':
+                wv.setEditTriggers(QAbstractItemView.EditTrigger.DoubleClicked|wv.editTriggers())
+            elif tval == 'open_viewer':
+                wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
                 wv.doubleClicked.connect(parent.iactions['View'].view_triggered)
-            elif tweaks['doubleclick_on_library_view'] == 'edit_metadata':
+            elif tval == 'show_book_details':
+                wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
+                wv.doubleClicked.connect(parent.iactions['Show Book Details'].show_book_info)
+            elif tval == 'edit_metadata':
                 # Must not enable single-click to edit, or the field will remain
                 # open in edit mode underneath the edit metadata dialog
                 if use_edit_metadata_dialog:
@@ -249,7 +256,7 @@ class BooksView(QTableView):  # {{{
                             partial(parent.iactions['Edit Metadata'].edit_metadata,
                                     checked=False))
                 else:
-                    wv.setEditTriggers(self.DoubleClicked|wv.editTriggers())
+                    wv.setEditTriggers(QAbstractItemView.EditTrigger.DoubleClicked|wv.editTriggers())
 
         setup_dnd_interface(self)
         for wv in self, self.pin_view:
@@ -283,19 +290,20 @@ class BooksView(QTableView):  # {{{
         self.setModel(self._model)
         self.pin_view.setModel(self._model)
         self._model.count_changed_signal.connect(self.do_row_sizing,
-                                                 type=Qt.QueuedConnection)
+                                                 type=Qt.ConnectionType.QueuedConnection)
         for wv in self, self.pin_view:
-            wv.setSelectionBehavior(QAbstractItemView.SelectRows)
+            wv.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
             wv.setSortingEnabled(True)
         self.selectionModel().currentRowChanged.connect(self._model.current_changed)
+        self.selectionModel().selectionChanged.connect(self.selection_changed.emit)
         self.preserve_state = partial(PreserveViewState, self)
         self.marked_changed_listener = FunctionDispatcher(self.marked_changed)
 
         # {{{ Column Header setup
         self.can_add_columns = True
         self.was_restored = False
-        self.column_header = HeaderView(Qt.Horizontal, self)
-        self.pin_view.column_header = HeaderView(Qt.Horizontal, self.pin_view)
+        self.column_header = HeaderView(Qt.Orientation.Horizontal, self)
+        self.pin_view.column_header = HeaderView(Qt.Orientation.Horizontal, self.pin_view)
         self.setHorizontalHeader(self.column_header)
         self.pin_view.setHorizontalHeader(self.pin_view.column_header)
         self.column_header.sectionMoved.connect(self.save_state)
@@ -304,24 +312,25 @@ class BooksView(QTableView):  # {{{
         self.pin_view.column_header.sortIndicatorChanged.disconnect()
         self.pin_view.column_header.sortIndicatorChanged.connect(self.pin_view_user_sort_requested)
         self.column_header.customContextMenuRequested.connect(partial(self.show_column_header_context_menu, view=self))
-        self.column_header.sectionResized.connect(self.column_resized, Qt.QueuedConnection)
+        self.column_header.sectionResized.connect(self.column_resized, Qt.ConnectionType.QueuedConnection)
         if self.is_library_view:
-            self.pin_view.column_header.sectionResized.connect(self.pin_view_column_resized, Qt.QueuedConnection)
+            self.pin_view.column_header.sectionResized.connect(self.pin_view_column_resized, Qt.ConnectionType.QueuedConnection)
             self.pin_view.column_header.sectionMoved.connect(self.pin_view.save_state)
             self.pin_view.column_header.customContextMenuRequested.connect(partial(self.show_column_header_context_menu, view=self.pin_view))
-        self.row_header = HeaderView(Qt.Vertical, self)
-        self.row_header.setSectionResizeMode(self.row_header.Fixed)
+        self.row_header = HeaderView(Qt.Orientation.Vertical, self)
+        self.row_header.setSectionResizeMode(QHeaderView.ResizeMode.Fixed)
+        self.row_header.customContextMenuRequested.connect(self.show_row_header_context_menu)
         self.setVerticalHeader(self.row_header)
         # }}}
 
         self._model.database_changed.connect(self.database_changed)
         hv = self.verticalHeader()
         hv.setSectionsClickable(True)
-        hv.setCursor(Qt.PointingHandCursor)
+        hv.setCursor(Qt.CursorShape.PointingHandCursor)
         self.selected_ids = []
         self._model.about_to_be_sorted.connect(self.about_to_be_sorted)
         self._model.sorting_done.connect(self.sorting_done,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.set_row_header_visibility()
         self.allow_mirroring = True
         if self.is_library_view:
@@ -337,7 +346,7 @@ class BooksView(QTableView):  # {{{
     # Pin view {{{
     def set_pin_view_visibility(self, visible=False):
         self.pin_view.setVisible(visible)
-        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff if visible else Qt.ScrollBarAsNeeded)
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff if visible else Qt.ScrollBarPolicy.ScrollBarAsNeeded)
         self.mirror_selection_between_views(self)
 
     def mirror_selection_between_views(self, src):
@@ -346,14 +355,14 @@ class BooksView(QTableView):  # {{{
             if dest is self.pin_view and not dest.isVisible():
                 return
             self.allow_mirroring = False
-            dest.selectionModel().select(src.selectionModel().selection(), QItemSelectionModel.ClearAndSelect)
+            dest.selectionModel().select(src.selectionModel().selection(), QItemSelectionModel.SelectionFlag.ClearAndSelect)
             ci = dest.currentIndex()
             nci = src.selectionModel().currentIndex()
             # Save/restore horz scroll.  ci column may be scrolled out of view.
             hpos = dest.horizontalScrollBar().value()
             if ci.isValid():
                 nci = dest.model().index(nci.row(), ci.column())
-            dest.selectionModel().setCurrentIndex(nci, QItemSelectionModel.NoUpdate)
+            dest.selectionModel().setCurrentIndex(nci, QItemSelectionModel.SelectionFlag.NoUpdate)
             dest.horizontalScrollBar().setValue(hpos)
             self.allow_mirroring = True
 
@@ -469,7 +478,8 @@ class BooksView(QTableView):  # {{{
         ans.addSeparator()
         if hidden_cols:
             m = ans.addMenu(_('Show column'))
-            hcols = [(hcol, unicode_type(self.model().headerData(hidx, Qt.Horizontal, Qt.DisplayRole) or '')) for hcol, hidx in iteritems(hidden_cols)]
+            hcols = [(hcol, unicode_type(self.model().headerData(hidx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or ''))
+                     for hcol, hidx in iteritems(hidden_cols)]
             hcols.sort(key=lambda x: primary_sort_key(x[1]))
             for hcol, hname in hcols:
                 m.addAction(hname, partial(handler, action='show', column=hcol))
@@ -489,13 +499,22 @@ class BooksView(QTableView):  # {{{
                     QIcon(I('column.png')), _('Add your own columns'), partial(handler, action='addcustcol'))
         return ans
 
+    def show_row_header_context_menu(self, pos):
+        menu = QMenu(self)
+        menu.addAction(_('Hide row numbers'), self.hide_row_numbers)
+        menu.popup(self.mapToGlobal(pos))
+
+    def hide_row_numbers(self):
+        gprefs['row_numbers_in_book_list'] = False
+        self.set_row_header_visibility()
+
     def show_column_header_context_menu(self, pos, view=None):
         view = view or self
         idx = view.column_header.logicalIndexAt(pos)
         col = None
         if idx > -1 and idx < len(self.column_map):
             col = self.column_map[idx]
-            name = unicode_type(self.model().headerData(idx, Qt.Horizontal, Qt.DisplayRole) or '')
+            name = unicode_type(self.model().headerData(idx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or '')
             view.column_header_context_menu = self.create_context_menu(col, name, view)
         has_context_menu = hasattr(view, 'column_header_context_menu')
         if self.is_library_view and has_context_menu:
@@ -519,11 +538,11 @@ class BooksView(QTableView):  # {{{
         for v in views:
             ch = v.column_header
             ch.blockSignals(True)
-            ch.setSortIndicator(logical_idx, Qt.AscendingOrder if ascending else Qt.DescendingOrder)
+            ch.setSortIndicator(logical_idx, Qt.SortOrder.AscendingOrder if ascending else Qt.SortOrder.DescendingOrder)
             ch.blockSignals(False)
 
     def sort_by_column_and_order(self, col, ascending):
-        order = Qt.AscendingOrder if ascending else Qt.DescendingOrder
+        order = Qt.SortOrder.AscendingOrder if ascending else Qt.SortOrder.DescendingOrder
         self.column_header.blockSignals(True)
         self.column_header.setSortIndicator(col, order)
         self.column_header.blockSignals(False)
@@ -531,15 +550,15 @@ class BooksView(QTableView):  # {{{
         if self.is_library_view:
             self.set_sort_indicator(col, ascending)
 
-    def user_sort_requested(self, col, order=Qt.AscendingOrder):
+    def user_sort_requested(self, col, order=Qt.SortOrder.AscendingOrder):
         if 0 <= col < len(self.column_map):
             field = self.column_map[col]
-            self.intelligent_sort(field, order == Qt.AscendingOrder)
+            self.intelligent_sort(field, order == Qt.SortOrder.AscendingOrder)
 
-    def pin_view_user_sort_requested(self, col, order=Qt.AscendingOrder):
+    def pin_view_user_sort_requested(self, col, order=Qt.SortOrder.AscendingOrder):
         if col < len(self.column_map) and col >= 0:
             field = self.column_map[col]
-            self.intelligent_sort(field, order == Qt.AscendingOrder)
+            self.intelligent_sort(field, order == Qt.SortOrder.AscendingOrder)
 
     def intelligent_sort(self, field, ascending):
         m = self.model()
@@ -895,14 +914,14 @@ class BooksView(QTableView):  # {{{
                     pass
             sections = tuple(x for x in map(f, changed) if x is not None)
             if sections:
-                self.row_header.headerDataChanged(Qt.Vertical, min(sections), max(sections))
+                self.row_header.headerDataChanged(Qt.Orientation.Vertical, min(sections), max(sections))
                 # This is needed otherwise Qt does not always update the
                 # viewport correctly. See https://bugs.launchpad.net/bugs/1404697
                 self.row_header.viewport().update()
         else:
             # Marked items have either appeared or all been removed
             self.model().set_row_decoration(current_marked)
-            self.row_header.headerDataChanged(Qt.Vertical, 0, self.row_header.count()-1)
+            self.row_header.headerDataChanged(Qt.Orientation.Vertical, 0, self.row_header.count()-1)
             self.row_header.geometriesChanged.emit()
             self.set_row_header_visibility()
 
@@ -971,13 +990,13 @@ class BooksView(QTableView):  # {{{
         self.set_ondevice_column_visibility()
         # incase there were marked books
         self.model().set_row_decoration(set())
-        self.row_header.headerDataChanged(Qt.Vertical, 0, self.row_header.count()-1)
+        self.row_header.headerDataChanged(Qt.Orientation.Vertical, 0, self.row_header.count()-1)
         self.row_header.geometriesChanged.emit()
         # }}}
 
     # Context Menu {{{
     def set_context_menu(self, menu, edit_collections_action):
-        self.setContextMenuPolicy(Qt.DefaultContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.DefaultContextMenu)
         self.context_menu = menu
         self.alternate_views.set_context_menu(menu)
         self.edit_collections_action = edit_collections_action
@@ -993,7 +1012,7 @@ class BooksView(QTableView):  # {{{
     # }}}
 
     def handle_mouse_press_event(self, ev):
-        if QApplication.keyboardModifiers() & Qt.ShiftModifier:
+        if QApplication.keyboardModifiers() & Qt.KeyboardModifier.ShiftModifier:
             # Shift-Click in QTableView is badly behaved.
             index = self.indexAt(ev.pos())
             if not index.isValid():
@@ -1005,11 +1024,14 @@ class BooksView(QTableView):  # {{{
             current_row = ci.row()
             sm = self.selectionModel()
             if clicked_row == current_row:
-                sm.setCurrentIndex(index, sm.NoUpdate)
+                sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
                 return
             sr = sm.selectedRows()
             if not len(sr):
-                sm.select(index, sm.Select | sm.Clear | sm.Current | sm.Rows)
+                sm.select(
+                    index,
+                    QItemSelectionModel.SelectionFlag.Select | QItemSelectionModel.SelectionFlag.Clear |
+                    QItemSelectionModel.SelectionFlag.Current | QItemSelectionModel.SelectionFlag.Rows)
                 return
 
             m = self.model()
@@ -1030,15 +1052,17 @@ class BooksView(QTableView):  # {{{
                     upper, lower = clicked_row, min_row
                 else:
                     upper, lower = max_row, clicked_row
-                existing_selection.merge(new_selection(upper, lower), sm.Select)
+                existing_selection.merge(new_selection(upper, lower), QItemSelectionModel.SelectionFlag.Select)
             else:
                 if current_row < clicked_row:
                     upper, lower = current_row, clicked_row
                 else:
                     upper, lower  = clicked_row, current_row
-                existing_selection.merge(new_selection(upper, lower), sm.Toggle)
-            sm.select(existing_selection, sm.ClearAndSelect)
-            sm.setCurrentIndex(index, sm.Select | sm.Rows)  # ensure clicked row is always selected
+                existing_selection.merge(new_selection(upper, lower), QItemSelectionModel.SelectionFlag.Toggle)
+            sm.select(existing_selection, QItemSelectionModel.SelectionFlag.ClearAndSelect)
+            sm.setCurrentIndex(
+                # ensure clicked row is always selected
+                index, QItemSelectionModel.SelectionFlag.Select | QItemSelectionModel.SelectionFlag.Rows)
         else:
             return QTableView.mousePressEvent(self, ev)
 
@@ -1078,7 +1102,7 @@ class BooksView(QTableView):  # {{{
             h = self.horizontalHeader()
             for i in range(h.count()):
                 if not h.isSectionHidden(i) and h.sectionViewportPosition(i) >= 0:
-                    self.scrollTo(self.model().index(row, i), self.PositionAtCenter)
+                    self.scrollTo(self.model().index(row, i), QAbstractItemView.ScrollHint.PositionAtCenter)
                     break
 
     @property
@@ -1120,20 +1144,20 @@ class BooksView(QTableView):  # {{{
             index = self.model().index(row, i)
             if for_sync:
                 sm = self.selectionModel()
-                sm.setCurrentIndex(index, sm.NoUpdate)
+                sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
             else:
                 self.setCurrentIndex(index)
                 if select:
                     sm = self.selectionModel()
-                    sm.select(index, sm.ClearAndSelect|sm.Rows)
+                    sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect|QItemSelectionModel.SelectionFlag.Rows)
 
     def select_cell(self, row_number=0, logical_column=0):
         if row_number > -1 and row_number < self.model().rowCount(QModelIndex()):
             index = self.model().index(row_number, logical_column)
             self.setCurrentIndex(index)
             sm = self.selectionModel()
-            sm.select(index, sm.ClearAndSelect|sm.Rows)
-            sm.select(index, sm.Current)
+            sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect|QItemSelectionModel.SelectionFlag.Rows)
+            sm.select(index, QItemSelectionModel.SelectionFlag.Current)
             self.clicked.emit(index)
 
     def row_at_top(self):
@@ -1156,7 +1180,7 @@ class BooksView(QTableView):  # {{{
     def moveCursor(self, action, modifiers):
         orig = self.currentIndex()
         index = QTableView.moveCursor(self, action, modifiers)
-        if action == QTableView.MovePageDown:
+        if action == QAbstractItemView.CursorAction.MovePageDown:
             moved = index.row() - orig.row()
             try:
                 rows = self.row_at_bottom() - self.row_at_top()
@@ -1164,7 +1188,7 @@ class BooksView(QTableView):  # {{{
                 rows = moved
             if moved > rows:
                 index = self.model().index(orig.row() + rows, index.column())
-        elif action == QTableView.MovePageUp:
+        elif action == QAbstractItemView.CursorAction.MovePageUp:
             moved = orig.row() - index.row()
             try:
                 rows = self.row_at_bottom() - self.row_at_top()
@@ -1172,15 +1196,15 @@ class BooksView(QTableView):  # {{{
                 rows = moved
             if moved > rows:
                 index = self.model().index(orig.row() - rows, index.column())
-        elif action == QTableView.MoveHome and modifiers & Qt.ControlModifier:
+        elif action == QAbstractItemView.CursorAction.MoveHome and modifiers & Qt.KeyboardModifier.ControlModifier:
             return self.model().index(0, orig.column())
-        elif action == QTableView.MoveEnd and modifiers & Qt.ControlModifier:
+        elif action == QAbstractItemView.CursorAction.MoveEnd and modifiers & Qt.KeyboardModifier.ControlModifier:
             return self.model().index(self.model().rowCount(QModelIndex()) - 1, orig.column())
         return index
 
     def selectionCommand(self, index, event):
-        if event and event.type() == event.KeyPress and event.key() in (Qt.Key_Home, Qt.Key_End) and event.modifiers() & Qt.CTRL:
-            return QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows
+        if event and event.type() == QEvent.Type.KeyPress and event.key() in (Qt.Key.Key_Home, Qt.Key.Key_End) and event.modifiers() & Qt.Modifier.CTRL:
+            return QItemSelectionModel.SelectionFlag.ClearAndSelect | QItemSelectionModel.SelectionFlag.Rows
         return super(BooksView, self).selectionCommand(index, event)
 
     def keyPressEvent(self, ev):
@@ -1232,18 +1256,21 @@ class BooksView(QTableView):  # {{{
         for k, g in itertools.groupby(enumerate(rows), lambda i_x:i_x[0]-i_x[1]):
             group = list(map(operator.itemgetter(1), g))
             sel.merge(QItemSelection(m.index(min(group), 0),
-                m.index(max(group), max_col)), sm.Select)
-        sm.select(sel, sm.ClearAndSelect)
+                m.index(max(group), max_col)), QItemSelectionModel.SelectionFlag.Select)
+        sm.select(sel, QItemSelectionModel.SelectionFlag.ClearAndSelect)
+        return rows
 
-    def get_selected_ids(self):
+    def get_selected_ids(self, as_set=False):
         ans = []
+        seen = set()
         m = self.model()
         for idx in self.selectedIndexes():
             r = idx.row()
             i = m.id(r)
-            if i not in ans:
+            if i not in seen:
                 ans.append(i)
-        return ans
+                seen.add(i)
+        return seen if as_set else ans
 
     @property
     def current_id(self):
@@ -1263,6 +1290,14 @@ class BooksView(QTableView):  # {{{
                 self.set_current_row(row, select=False)
                 break
 
+    def show_next_book(self):
+        ci = self.currentIndex()
+        if not ci.isValid():
+            self.set_current_row()
+            return
+        n = (ci.row() + 1) % self.model().rowCount(QModelIndex())
+        self.set_current_row(n)
+
     @property
     def next_id(self):
         '''
@@ -1323,7 +1358,7 @@ class BooksView(QTableView):  # {{{
         elif self._model.highlight_only:
             self.clearSelection()
         if self.isVisible() and getattr(txt, 'as_you_type', False) is not True:
-            self.setFocus(Qt.OtherFocusReason)
+            self.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def connect_to_search_box(self, sb, search_done):
         sb.search.connect(self.search_proxy)
@@ -1352,14 +1387,14 @@ class DeviceBooksView(BooksView):  # {{{
         BooksView.__init__(self, parent, DeviceBooksModel,
                            use_edit_metadata_dialog=False)
         self._model.resize_rows.connect(self.do_row_sizing,
-                                                 type=Qt.QueuedConnection)
+                                                 type=Qt.ConnectionType.QueuedConnection)
         self.can_add_columns = False
         self.resize_on_select = False
         self.rating_delegate = None
         self.half_rating_delegate = None
         for i in range(10):
             self.setItemDelegateForColumn(i, TextDelegate(self))
-        self.setDragDropMode(self.NoDragDrop)
+        self.setDragDropMode(QAbstractItemView.DragDropMode.NoDragDrop)
         self.setAcceptDrops(False)
         self.set_row_header_visibility()
 
