@@ -6,28 +6,33 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import traceback, copy, os
-from collections import OrderedDict
-
-from PyQt5.Qt import (QAbstractItemModel, QIcon, QFont, Qt,
-        QMimeData, QModelIndex, pyqtSignal, QObject)
+import copy
+import os
+import traceback
+from collections import OrderedDict, namedtuple
+from qt.core import (
+    QAbstractItemModel, QFont, QIcon, QMimeData, QModelIndex, QObject, Qt,
+    pyqtSignal
+)
 
 from calibre.constants import config_dir
-from calibre.ebooks.metadata import rating_to_stars
-from calibre.gui2 import gprefs, config, error_dialog, file_icon_provider
 from calibre.db.categories import Tag
-from calibre.utils.config import tweaks, prefs
-from calibre.utils.icu import sort_key, lower, strcmp, collation_order, primary_strcmp, primary_contains, contains
-from calibre.library.field_metadata import category_icon_map
+from calibre.ebooks.metadata import rating_to_stars
+from calibre.gui2 import config, error_dialog, file_icon_provider, gprefs, question_dialog
 from calibre.gui2.dialogs.confirm_delete import confirm
+from calibre.library.field_metadata import category_icon_map
+from calibre.utils.config import prefs, tweaks
 from calibre.utils.formatter import EvalFormatter
+from calibre.utils.icu import (
+    collation_order, contains, lower, primary_contains, primary_strcmp, sort_key,
+    strcmp
+)
 from calibre.utils.serialize import json_dumps, json_loads
 from polyglot.builtins import iteritems, itervalues, map, range, unicode_type
 
-
 TAG_SEARCH_STATES = {'clear': 0, 'mark_plus': 1, 'mark_plusplus': 2,
                      'mark_minus': 3, 'mark_minusminus': 4}
-DRAG_IMAGE_ROLE = Qt.UserRole + 1000
+DRAG_IMAGE_ROLE = Qt.ItemDataRole.UserRole + 1000
 COUNT_ROLE = DRAG_IMAGE_ROLE + 1
 
 _bf = None
@@ -156,7 +161,7 @@ class TagTreeItem(object):  # {{{
         return self.cached_item_count
 
     def data(self, role):
-        if role == Qt.UserRole:
+        if role == Qt.ItemDataRole.UserRole:
             return self
         if self.type == self.TAG:
             return self.tag_data(role)
@@ -165,17 +170,17 @@ class TagTreeItem(object):  # {{{
         return None
 
     def category_data(self, role):
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return self.py_name
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             return (self.py_name)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             if not self.tag.state:
                 self.ensure_icon()
             return self.icon_state_map[self.tag.state]
-        if role == Qt.FontRole:
+        if role == Qt.ItemDataRole.FontRole:
             return bf()
-        if role == Qt.ToolTipRole:
+        if role == Qt.ItemDataRole.ToolTipRole:
             return self.tooltip if gprefs['tag_browser_show_tooltips'] else None
         if role == DRAG_IMAGE_ROLE:
             self.ensure_icon()
@@ -193,15 +198,15 @@ class TagTreeItem(object):  # {{{
                 name = tag.original_name
             else:
                 name = tag.name
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return unicode_type(name)
-        if role == Qt.EditRole:
+        if role == Qt.ItemDataRole.EditRole:
             return (tag.original_name)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             if not tag.state:
                 self.ensure_icon()
             return self.icon_state_map[tag.state]
-        if role == Qt.ToolTipRole:
+        if role == Qt.ItemDataRole.ToolTipRole:
             if gprefs['tag_browser_show_tooltips']:
                 tt = [self.tooltip] if self.tooltip else []
                 if tag.original_categories:
@@ -211,7 +216,7 @@ class TagTreeItem(object):  # {{{
                 ar = self.average_rating
                 if ar:
                     tt.append(_('Average rating for books in this category: %.1f') % ar)
-                elif self.type == self.TAG and ar is not None:
+                elif self.type == self.TAG and ar is not None and self.tag.category != 'search':
                     tt.append(_('Books in this category are unrated'))
                 if self.type == self.TAG and self.tag.category == 'search':
                     tt.append(_('Search expression:') + ' ' + self.tag.search_expression)
@@ -290,15 +295,28 @@ class TagTreeItem(object):  # {{{
     # }}}
 
 
+FL_Interval = namedtuple('FL_Interval', ('first_chr', 'last_chr', 'length'))
+
+
+def rename_only_in_vl_question(parent):
+    return question_dialog(parent,
+                           _('Rename in Virtual library'), '<p>' +
+                           _('Do you want this rename to apply only to books '
+                             'in the current Virtual library?') + '</p>',
+                           yes_text=_('Yes, apply only in VL'),
+                           no_text=_('No, apply in entire library'))
+
+
 class TagsModel(QAbstractItemModel):  # {{{
 
     search_item_renamed = pyqtSignal()
     tag_item_renamed = pyqtSignal()
     refresh_required = pyqtSignal()
-    restriction_error = pyqtSignal()
+    restriction_error = pyqtSignal(object)
     drag_drop_finished = pyqtSignal(object)
     user_categories_edited = pyqtSignal(object, object)
     user_category_added = pyqtSignal()
+    show_error_after_event_loop_tick_signal = pyqtSignal(object, object, object)
 
     def __init__(self, parent, prefs=gprefs):
         QAbstractItemModel.__init__(self, parent)
@@ -324,6 +342,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         self.db = None
         self._build_in_progress = False
         self.reread_collapse_model({}, rebuild=False)
+        self.show_error_after_event_loop_tick_signal.connect(self.on_show_error_after_event_loop_tick, type=Qt.ConnectionType.QueuedConnection)
 
     @property
     def gui_parent(self):
@@ -525,18 +544,72 @@ class TagsModel(QAbstractItemModel):  # {{{
                 # Build a list of 'equal' first letters by noticing changes
                 # in ICU's 'ordinal' for the first letter. In this case, the
                 # first letter can actually be more than one letter long.
+                fl_collapse_when = self.prefs['tags_browser_collapse_fl_at']
+                fl_collapse = True if fl_collapse_when > 1 else False
+                intervals = []
                 cl_list = [None] * len(data[key])
                 last_ordnum = 0
                 last_c = ' '
+                last_idx = 0
                 for idx,tag in enumerate(data[key]):
                     # Deal with items that don't have sorts, such as formats
                     t = tag.sort if tag.sort else tag.name
                     c = icu_upper(t) if t else ' '
                     ordnum, ordlen = collation_order(c)
                     if last_ordnum != ordnum:
+                        if fl_collapse and idx > 0:
+                            intervals.append(FL_Interval(last_c, last_c, idx-last_idx))
+                            last_idx = idx
                         last_c = c[0:ordlen]
                         last_ordnum = ordnum
                     cl_list[idx] = last_c
+                if fl_collapse:
+                    intervals.append(FL_Interval(last_c, last_c, len(cl_list)-last_idx))
+                    # Combine together first letter categories that are smaller
+                    # than the specified option. We choose which item to combine
+                    # by the size of the items before and after, privileging making
+                    # smaller categories. Loop through the intervals doing the combine
+                    # until nothing changes. Multiple iterations are required because
+                    # we might need to combine categories that are already combined.
+                    fl_intervals_changed = True
+                    null_interval = FL_Interval('', '', 100000000)
+                    while fl_intervals_changed and len(intervals) > 1:
+                        fl_intervals_changed = False
+                        for idx,interval in enumerate(intervals):
+                            if interval.length >= fl_collapse_when:
+                                continue
+                            prev = next_ = null_interval
+                            if idx == 0:
+                                next_ = intervals[idx+1]
+                            else:
+                                prev = intervals[idx-1]
+                                if idx < len(intervals) - 1:
+                                    next_ = intervals[idx+1]
+                            if prev.length < next_.length:
+                                intervals[idx-1] = FL_Interval(prev.first_chr,
+                                                               interval.last_chr,
+                                                               prev.length + interval.length)
+                            else:
+                                intervals[idx+1] = FL_Interval(interval.first_chr,
+                                                               next_.last_chr,
+                                                               next_.length + interval.length)
+                            del intervals[idx]
+                            fl_intervals_changed = True
+                            break
+                    # Now correct the first letter list, entering either the letter
+                    # or the range for each item in the category. If we ended up
+                    # with only one 'first letter' category then don't combine
+                    # letters and revert to basic 'by first letter'
+                    if len(intervals) > 1:
+                        cur_idx = 0
+                        for interval in intervals:
+                            first_chr, last_chr, length = interval
+                            for i in range(0, length):
+                                if first_chr == last_chr:
+                                    cl_list[cur_idx] = first_chr
+                                else:
+                                    cl_list[cur_idx] = '{0} - {1}'.format(first_chr, last_chr)
+                                cur_idx += 1
             top_level_component = 'z' + data[key][0].original_name
 
             last_idx = -collapse
@@ -771,7 +844,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         if not fmts.intersection(set(self.mimeTypes())):
             return False
         if "application/calibre+from_library" in fmts:
-            if action != Qt.CopyAction:
+            if action != Qt.DropAction.CopyAction:
                 return False
             return self.do_drop_from_library(md, action, row, column, parent)
         elif 'application/calibre+from_tag_browser' in fmts:
@@ -797,7 +870,7 @@ class TagsModel(QAbstractItemModel):  # {{{
                 # dropped on itself
                 return False
             src_item = self.get_node(src_index)
-            dest_item = parent.data(Qt.UserRole)
+            dest_item = parent.data(Qt.ItemDataRole.UserRole)
             # Here we do the real work. If src is a tag, src == dest, and src
             # is hierarchical then we can do a rename.
             if (src_item.type == TagTreeItem.TAG and
@@ -814,8 +887,10 @@ class TagsModel(QAbstractItemModel):  # {{{
                     new_name = dest_item.tag.original_name + '.' + src_simple_name
                 else:
                     new_name = src_simple_name
-                # In d&d renames always use the vl. This might be controversial.
-                src_item.use_vl = True
+                if self.get_in_vl():
+                    src_item.use_vl = rename_only_in_vl_question(self.gui_parent)
+                else:
+                    src_item.use_vl = False
                 self.rename_item(src_item, key, new_name)
                 return True
         # Should be working with a user category
@@ -830,7 +905,7 @@ class TagsModel(QAbstractItemModel):  # {{{
          full name, category key, path to node)
         The type must be TagTreeItem.TAG
         dest is the TagTreeItem node to receive the items
-        action is Qt.CopyAction or Qt.MoveAction
+        action is Qt.DropAction.CopyAction or Qt.DropAction.MoveAction
         '''
         def process_source_node(user_cats, src_parent, src_parent_is_gst,
                                 is_uc, dest_key, idx):
@@ -842,7 +917,7 @@ class TagsModel(QAbstractItemModel):  # {{{
             src_cat = idx.tag.category
             # delete the item if the source is a User category and action is move
             if is_uc and not src_parent_is_gst and src_parent in user_cats and \
-                                    action == Qt.MoveAction:
+                                    action == Qt.DropAction.MoveAction:
                 new_cat = []
                 for tup in user_cats[src_parent]:
                     if src_name == tup[0] and src_cat == tup[1]:
@@ -898,7 +973,7 @@ class TagsModel(QAbstractItemModel):  # {{{
     def do_drop_from_library(self, md, action, row, column, parent):
         idx = parent
         if idx.isValid():
-            node = self.data(idx, Qt.UserRole)
+            node = self.data(idx, Qt.ItemDataRole.UserRole)
             if node.type == TagTreeItem.TAG:
                 fm = self.db.metadata_for_field(node.tag.category)
                 if node.tag.category in \
@@ -1038,12 +1113,11 @@ class TagsModel(QAbstractItemModel):  # {{{
             data = self.db.new_api.get_categories(sort=sort,
                     book_ids=self.get_book_ids_to_use(),
                     first_letter_sort=self.collapse_model == 'first letter')
-        except:
-            import traceback
+        except Exception as e:
             traceback.print_exc()
             data = self.db.new_api.get_categories(sort=sort,
                     first_letter_sort=self.collapse_model == 'first letter')
-            self.restriction_error.emit()
+            self.restriction_error.emit(str(e))
 
         if self.filter_categories_by:
             if self.filter_categories_by.startswith('='):
@@ -1128,7 +1202,7 @@ class TagsModel(QAbstractItemModel):  # {{{
         item = self.get_node(index)
         return item.data(role)
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
         if not index.isValid():
             return False
         # set up to reposition at the same item. We can do this except if
@@ -1136,16 +1210,14 @@ class TagsModel(QAbstractItemModel):  # {{{
         # we position at the parent label
         val = unicode_type(value or '').strip()
         if not val:
-            error_dialog(self.gui_parent, _('Item is blank'),
-                        _('An item cannot be set to nothing. Delete it instead.')).exec_()
-            return False
+            return self.show_error_after_event_loop_tick(_('Item is blank'),
+                        _('An item cannot be set to nothing. Delete it instead.'))
         item = self.get_node(index)
         if item.type == TagTreeItem.CATEGORY and item.category_key.startswith('@'):
             if val.find('.') >= 0:
-                error_dialog(self.gui_parent, _('Rename User category'),
+                return self.show_error_after_event_loop_tick(_('Rename User category'),
                     _('You cannot use periods in the name when '
-                      'renaming User categories'), show=True)
-                return False
+                      'renaming User categories'))
 
             user_cats = self.db.new_api.pref('user_categories', {})
             user_cat_keys_lower = [icu_lower(k) for k in user_cats]
@@ -1167,18 +1239,16 @@ class TagsModel(QAbstractItemModel):  # {{{
                     if len(c) == len(ckey):
                         if strcmp(ckey, nkey) != 0 and \
                                 nkey_lower in user_cat_keys_lower:
-                            error_dialog(self.gui_parent, _('Rename User category'),
-                                _('The name %s is already used')%nkey, show=True)
-                            return False
+                            return self.show_error_after_event_loop_tick(_('Rename User category'),
+                                _('The name %s is already used')%nkey)
                         user_cats[nkey] = user_cats[ckey]
                         del user_cats[ckey]
                     elif c[len(ckey)] == '.':
                         rest = c[len(ckey):]
                         if strcmp(ckey, nkey) != 0 and \
                                     icu_lower(nkey + rest) in user_cat_keys_lower:
-                            error_dialog(self.gui_parent, _('Rename User category'),
-                                _('The name %s is already used')%(nkey+rest), show=True)
-                            return False
+                            return self.show_error_after_event_loop_tick(_('Rename User category'),
+                                _('The name %s is already used')%(nkey+rest))
                         user_cats[nkey + rest] = user_cats[ckey + rest]
                         del user_cats[ckey + rest]
             self.user_categories_edited.emit(user_cats, nkey)  # Does a refresh
@@ -1191,14 +1261,13 @@ class TagsModel(QAbstractItemModel):  # {{{
             return False
         if key == 'authors':
             if val.find('&') >= 0:
-                error_dialog(self.gui_parent, _('Invalid author name'),
-                        _('Author names cannot contain & characters.')).exec_()
+                return self.show_error_after_event_loop_tick(_('Invalid author name'),
+                        _('Author names cannot contain & characters.'))
                 return False
         if key == 'search':
             if val in self.db.saved_search_names():
-                error_dialog(self.gui_parent, _('Duplicate search name'),
-                    _('The saved search name %s is already used.')%val).exec_()
-                return False
+                return self.show_error_after_event_loop_tick(
+                    _('Duplicate search name'), _('The saved search name %s is already used.')%val)
             self.use_position_based_index_on_next_recount = True
             self.db.saved_search_rename(unicode_type(item.data(role) or ''), val)
             item.tag.name = val
@@ -1207,6 +1276,13 @@ class TagsModel(QAbstractItemModel):  # {{{
             self.rename_item(item, key, val)
         return True
 
+    def show_error_after_event_loop_tick(self, title, msg, det_msg=''):
+        self.show_error_after_event_loop_tick_signal.emit(title, msg, det_msg)
+        return False
+
+    def on_show_error_after_event_loop_tick(self, title, msg, details):
+        error_dialog(self.gui_parent, title, msg, det_msg=details, show=True)
+
     def rename_item(self, item, key, to_what):
         def do_one_item(lookup_key, an_item, original_name, new_name, restrict_to_books):
             self.use_position_based_index_on_next_recount = True
@@ -1291,24 +1367,26 @@ class TagsModel(QAbstractItemModel):  # {{{
         return None
 
     def flags(self, index, *args):
-        ans = Qt.ItemIsEnabled|Qt.ItemIsEditable
+        ans = Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsEditable
         if index.isValid():
-            node = self.data(index, Qt.UserRole)
+            node = self.data(index, Qt.ItemDataRole.UserRole)
             if node.type == TagTreeItem.TAG:
-                if node.tag.is_editable or node.tag.is_hierarchical:
-                    ans |= Qt.ItemIsDragEnabled
-                fm = self.db.metadata_for_field(node.tag.category)
-                if node.tag.category in \
+                tag = node.tag
+                category = tag.category
+                if (tag.is_editable or tag.is_hierarchical) and category != 'search':
+                    ans |= Qt.ItemFlag.ItemIsDragEnabled
+                fm = self.db.metadata_for_field(category)
+                if category in \
                     ('tags', 'series', 'authors', 'rating', 'publisher', 'languages') or \
                     (fm['is_custom'] and
                         fm['datatype'] in ['text', 'rating', 'series', 'enumeration']):
-                    ans |= Qt.ItemIsDropEnabled
+                    ans |= Qt.ItemFlag.ItemIsDropEnabled
             else:
-                ans |= Qt.ItemIsDropEnabled
+                ans |= Qt.ItemFlag.ItemIsDropEnabled
         return ans
 
     def supportedDropActions(self):
-        return Qt.CopyAction|Qt.MoveAction
+        return Qt.DropAction.CopyAction|Qt.DropAction.MoveAction
 
     def named_path_for_index(self, index):
         ans = []
