@@ -1,20 +1,20 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, codecs, os
+import re, codecs, os, numbers
 from collections import namedtuple
-from types import StringType, UnicodeType
 
-from calibre import (strftime)
+from calibre import strftime
 from calibre.customize import CatalogPlugin
 from calibre.library.catalogs import FIELDS, TEMPLATE_ALLOWED_FIELDS
 from calibre.customize.conversion import DummyReporter
-from calibre.constants import preferred_encoding
 from calibre.ebooks.metadata import format_isbn
+from polyglot.builtins import filter, string_or_bytes, unicode_type
 
 
 class BIBTEX(CatalogPlugin):
@@ -23,11 +23,11 @@ class BIBTEX(CatalogPlugin):
     Option = namedtuple('Option', 'option, default, dest, action, help')
 
     name = 'Catalog_BIBTEX'
-    description = 'BIBTEX catalog generator'
+    description = _('BIBTEX catalog generator')
     supported_platforms = ['windows', 'osx', 'linux']
     author = 'Sengian'
     version = (1, 0, 0)
-    file_types = set(['bib'])
+    file_types = {'bib'}
 
     cli_options = [
             Option('--fields',
@@ -112,7 +112,6 @@ class BIBTEX(CatalogPlugin):
         from calibre.utils.html2text import html2text
         from calibre.utils.bibtex import BibTeX
         from calibre.library.save_to_disk import preprocess_template
-        from calibre.utils.date import now as nowf
         from calibre.utils.logging import default_log as log
         from calibre.utils.filenames import ascii_text
 
@@ -126,9 +125,9 @@ class BIBTEX(CatalogPlugin):
 
             bibtex_entry = []
             if mode != "misc" and check_entry_book_valid(entry) :
-                bibtex_entry.append(u'@book{')
+                bibtex_entry.append('@book{')
             elif mode != "book" :
-                bibtex_entry.append(u'@misc{')
+                bibtex_entry.append('@misc{')
             else :
                 # case strict book
                 return ''
@@ -137,12 +136,12 @@ class BIBTEX(CatalogPlugin):
                 # Citation tag
                 bibtex_entry.append(make_bibtex_citation(entry, template_citation,
                     bibtexdict))
-                bibtex_entry = [u' '.join(bibtex_entry)]
+                bibtex_entry = [' '.join(bibtex_entry)]
 
             for field in fields:
                 if field.startswith('#'):
                     item = db.get_field(entry['id'],field,index_is_id=True)
-                    if isinstance(item, (bool, float, int)):
+                    if isinstance(item, (bool, numbers.Number)):
                         item = repr(item)
                 elif field == 'title_sort':
                     item = entry['sort']
@@ -161,68 +160,68 @@ class BIBTEX(CatalogPlugin):
                     pass
 
                 if field == 'authors' :
-                    bibtex_entry.append(u'author = "%s"' % bibtexdict.bibtex_author_format(item))
+                    bibtex_entry.append('author = "%s"' % bibtexdict.bibtex_author_format(item))
 
                 elif field == 'id' :
-                    bibtex_entry.append(u'calibreid = "%s"' % int(item))
+                    bibtex_entry.append('calibreid = "%s"' % int(item))
 
                 elif field == 'rating' :
-                    bibtex_entry.append(u'rating = "%s"' % int(item))
+                    bibtex_entry.append('rating = "%s"' % int(item))
 
                 elif field == 'size' :
-                    bibtex_entry.append(u'%s = "%s octets"' % (field, int(item)))
+                    bibtex_entry.append('%s = "%s octets"' % (field, int(item)))
 
                 elif field == 'tags' :
                     # A list to flatten
-                    bibtex_entry.append(u'tags = "%s"' % bibtexdict.utf8ToBibtex(u', '.join(item)))
+                    bibtex_entry.append('tags = "%s"' % bibtexdict.utf8ToBibtex(', '.join(item)))
 
                 elif field == 'comments' :
                     # \n removal
-                    item = item.replace(u'\r\n',u' ')
-                    item = item.replace(u'\n',u' ')
+                    item = item.replace('\r\n', ' ')
+                    item = item.replace('\n', ' ')
                     # unmatched brace removal (users should use \leftbrace or \rightbrace for single braces)
-                    item = bibtexdict.stripUnmatchedSyntax(item, u'{', u'}')
+                    item = bibtexdict.stripUnmatchedSyntax(item, '{', '}')
                     # html to text
                     try:
                         item = html2text(item)
                     except:
                         log.warn("Failed to convert comments to text")
-                    bibtex_entry.append(u'note = "%s"' % bibtexdict.utf8ToBibtex(item))
+                    bibtex_entry.append('note = "%s"' % bibtexdict.utf8ToBibtex(item))
 
                 elif field == 'isbn' :
                     # Could be 9, 10 or 13 digits
-                    bibtex_entry.append(u'isbn = "%s"' % format_isbn(item))
+                    bibtex_entry.append('isbn = "%s"' % format_isbn(item))
 
                 elif field == 'formats' :
                     # Add file path if format is selected
                     formats = [format.rpartition('.')[2].lower() for format in item]
-                    bibtex_entry.append(u'formats = "%s"' % u', '.join(formats))
+                    bibtex_entry.append('formats = "%s"' % ', '.join(formats))
                     if calibre_files:
-                        files = [u':%s:%s' % (format, format.rpartition('.')[2].upper())
+                        files = [':%s:%s' % (format, format.rpartition('.')[2].upper())
                             for format in item]
-                        bibtex_entry.append(u'file = "%s"' % u', '.join(files))
+                        bibtex_entry.append('file = "%s"' % ', '.join(files))
 
                 elif field == 'series_index' :
-                    bibtex_entry.append(u'volume = "%s"' % int(item))
+                    bibtex_entry.append('volume = "%s"' % int(item))
 
                 elif field == 'timestamp' :
-                    bibtex_entry.append(u'timestamp = "%s"' % isoformat(item).partition('T')[0])
+                    bibtex_entry.append('timestamp = "%s"' % isoformat(item).partition('T')[0])
 
                 elif field == 'pubdate' :
-                    bibtex_entry.append(u'year = "%s"' % item.year)
-                    bibtex_entry.append(u'month = "%s"' % bibtexdict.utf8ToBibtex(strftime("%b", item)))
+                    bibtex_entry.append('year = "%s"' % item.year)
+                    bibtex_entry.append('month = "%s"' % bibtexdict.utf8ToBibtex(strftime("%b", item)))
 
-                elif field.startswith('#') and isinstance(item, basestring):
-                    bibtex_entry.append(u'custom_%s = "%s"' % (field[1:],
+                elif field.startswith('#') and isinstance(item, string_or_bytes):
+                    bibtex_entry.append('custom_%s = "%s"' % (field[1:],
                         bibtexdict.utf8ToBibtex(item)))
 
-                elif isinstance(item, basestring):
+                elif isinstance(item, string_or_bytes):
                     # elif field in ['title', 'publisher', 'cover', 'uuid', 'ondevice',
-                        # 'author_sort', 'series', 'title_sort'] :
-                    bibtex_entry.append(u'%s = "%s"' % (field, bibtexdict.utf8ToBibtex(item)))
+                    # 'author_sort', 'series', 'title_sort'] :
+                    bibtex_entry.append('%s = "%s"' % (field, bibtexdict.utf8ToBibtex(item)))
 
-            bibtex_entry = u',\n    '.join(bibtex_entry)
-            bibtex_entry += u' }\n\n'
+            bibtex_entry = ',\n    '.join(bibtex_entry)
+            bibtex_entry += ' }\n\n'
 
             return bibtex_entry
 
@@ -241,7 +240,7 @@ class BIBTEX(CatalogPlugin):
             # define a function to replace the template entry by its value
             def tpl_replace(objtplname) :
 
-                tpl_field = re.sub(u'[\{\}]', u'', objtplname.group())
+                tpl_field = re.sub(r'[\{\}]', '', objtplname.group())
 
                 if tpl_field in TEMPLATE_ALLOWED_FIELDS :
                     if tpl_field in ['pubdate', 'timestamp'] :
@@ -249,26 +248,26 @@ class BIBTEX(CatalogPlugin):
                     elif tpl_field in ['tags', 'authors'] :
                         tpl_field =entry[tpl_field][0]
                     elif tpl_field in ['id', 'series_index'] :
-                        tpl_field = str(entry[tpl_field])
+                        tpl_field = unicode_type(entry[tpl_field])
                     else :
                         tpl_field = entry[tpl_field]
                     return ascii_text(tpl_field)
                 else:
-                    return u''
+                    return ''
 
             if len(template_citation) >0 :
                 tpl_citation = bibtexclass.utf8ToBibtex(
-                    bibtexclass.ValidateCitationKey(re.sub(u'\{[^{}]*\}',
+                    bibtexclass.ValidateCitationKey(re.sub(r'\{[^{}]*\}',
                         tpl_replace, template_citation)))
 
                 if len(tpl_citation) >0 :
                     return tpl_citation
 
             if len(entry["isbn"]) > 0 :
-                template_citation = u'%s' % re.sub(u'[\D]',u'', entry["isbn"])
+                template_citation = '%s' % re.sub(r'[\D]','', entry["isbn"])
 
             else :
-                template_citation = u'%s' % str(entry["id"])
+                template_citation = '%s' % unicode_type(entry["id"])
 
             return bibtexclass.ValidateCitationKey(template_citation)
 
@@ -350,7 +349,7 @@ class BIBTEX(CatalogPlugin):
             bibtexc.ascii_bibtex = True
 
         # Check citation choice and go to default in case of bad CLI
-        if isinstance(opts.impcit, (StringType, UnicodeType)) :
+        if isinstance(opts.impcit, string_or_bytes) :
             if opts.impcit == 'False' :
                 citation_bibtex= False
             elif opts.impcit == 'True' :
@@ -362,7 +361,7 @@ class BIBTEX(CatalogPlugin):
             citation_bibtex= opts.impcit
 
         # Check add file entry and go to default in case of bad CLI
-        if isinstance(opts.addfiles, (StringType, UnicodeType)) :
+        if isinstance(opts.addfiles, string_or_bytes) :
             if opts.addfiles == 'False' :
                 addfiles_bibtex = False
             elif opts.addfiles == 'True' :
@@ -384,7 +383,7 @@ class BIBTEX(CatalogPlugin):
 
             # check in book strict if all is ok else throw a warning into log
             if bib_entry == 'book' :
-                nb_books = len(filter(check_entry_book_valid, data))
+                nb_books = len(list(filter(check_entry_book_valid, data)))
                 if nb_books < nb_entries :
                     log.warn("Only %d entries in %d are book compatible" % (nb_books, nb_entries))
                     nb_entries = nb_books
@@ -394,9 +393,9 @@ class BIBTEX(CatalogPlugin):
                 for entry in data:
                     entry['ondevice'] = db.catalog_plugin_on_device_temp_mapping[entry['id']]['ondevice']
 
-            outfile.write(u'%%%Calibre catalog\n%%%{0} entries in catalog\n\n'.format(nb_entries))
-            outfile.write(u'@preamble{"This catalog of %d entries was generated by calibre on %s"}\n\n'
-                % (nb_entries, nowf().strftime("%A, %d. %B %Y %H:%M").decode(preferred_encoding)))
+            # outfile.write('%%%Calibre catalog\n%%%{0} entries in catalog\n\n'.format(nb_entries))
+            outfile.write('@preamble{"This catalog of %d entries was generated by calibre on %s"}\n\n'
+                % (nb_entries, strftime("%A, %d. %B %Y %H:%M")))
 
             for entry in data:
                 outfile.write(create_bibtex_entry(entry, fields, bib_entry, template_citation,
